from http.cookies import SimpleCookie
from typing import Dict, Any, Callable

from inspira.auth.auth_utils import decode_auth_token
from inspira.auth.mixins.user_mixin import AnonymousUserMixin
from inspira.config import Config
from inspira.globals import get_global_app
from inspira.requests import RequestContext
from inspira.utils.session_utils import decode_session_data


class UserLoaderMiddleware:
    def __init__(self, user_model):
        self.user_model = user_model
        self.app = get_global_app()

    async def __call__(self, handler):
        async def middleware(scope: Dict[str, Any], receive: Callable, send: Callable):
            request = RequestContext().get_request()
            cookies = SimpleCookie(request.get_headers().get("cookie", ""))
            token = cookies.get(self.app.config["SESSION_COOKIE_NAME"])

            user = None

            if token:
                decoded_session = decode_session_data(token.value, self.app.secret_key)
                user_id = decode_auth_token(decoded_session.get("token", None))

                if user_id:
                    user = self.user_model.query.get(user_id)
            RequestContext.set_current_user(user or AnonymousUserMixin())
            request.user = RequestContext.get_current_user()

            await handler(scope, receive, send)

        return middleware



import os
import re

from inspira.utils import pluralize_word, singularize


def generate_repository_file(module_name):
    base_path = "src"
    # Specify the directory path
    model_directory = os.path.join(base_path, module_name.lower())

    template_path = os.path.join(
        os.path.dirname(__file__), "templates", "repository_template.txt"
    )
    repository_file_path = os.path.join(
        model_directory, f"{singularize(module_name.lower())}_repository.py"
    )

    with open(template_path, "r") as template_file, open(
        repository_file_path, "w"
    ) as output_file:
        content = (
            template_file.read()
            .replace("{{model_name}}", pluralize_word(module_name))
            .replace("{{model_name_upper}}", singularize(module_name.capitalize()))
            .replace("{{module_name}}", module_name)
            .replace("{{model_name_singular}}", singularize(module_name.lower()))
        )
        output_file.write(content)

    add_repository_dependency_to_service(module_name)


def add_repository_dependency_to_service(module_name):
    controller_file_path = os.path.join(
        "src", module_name.lower(), f"{singularize(module_name.lower())}_service.py"
    )
    module_name_capitalized = singularize(module_name.lower()).capitalize()
    singularized_module_name = singularize(module_name.lower())

    with open(controller_file_path, "r") as controller_file:
        existing_content = controller_file.read()

    import_statement = (
        f"from src.{module_name.lower()}.{singularized_module_name}_repository "
        f"import {module_name_capitalized}Repository\n\n\n"
    )

    init_method = (
        f"    def __init__(self, {singularized_module_name}_repository: {module_name_capitalized}Repository):"
        f"\n        self._{singularized_module_name}_repository = {singularized_module_name}_repository"
    )

    class_pattern = re.compile(r"class\s+[A-Za-z_]\w*:")

    match = class_pattern.search(existing_content)

    if match:
        start, end = match.span()
        updated_content = (
            existing_content[:start]
            + import_statement
            + existing_content[start:end].rstrip()
            + "\n"
            + init_method
            + existing_content[end:]
        )

        with open(controller_file_path, "w") as controller_file:
            controller_file.write(updated_content)



from typing import List, Dict, Any, Callable

from inspira.helpers.error_handlers import handle_forbidden
from inspira.requests import Request


class CORSMiddleware:
    def __init__(
        self,
        allow_origins: List[str] = None,
        allow_credentials: bool = False,
        allow_methods: List[str] = None,
        allow_headers: List[str] = None,
    ):
        self.allow_origins = allow_origins or []
        self.allow_credentials = allow_credentials
        self.allow_methods = allow_methods or []
        self.allow_headers = allow_headers or []

    async def __call__(self, handler):
        async def middleware(scope: Dict[str, Any], receive: Callable, send: Callable):
            request = Request(scope, receive, send)
            origin = request.get_headers().get("origin")

            if origin is not None and (
                origin not in self.allow_origins and "*" not in self.allow_origins
            ):
                return await handle_forbidden(scope, receive, send)

            async def send_wrapper(message):
                if message["type"] == "http.response.start":
                    headers = message.get("headers", [])

                    if origin in self.allow_origins:
                        headers.append(
                            (b"Access-Control-Allow-Origin", origin.encode())
                        )
                        headers.append(
                            (
                                b"Access-Control-Allow-Credentials",
                                str(self.allow_credentials).lower().encode(),
                            )
                        )
                        headers.append(
                            (
                                b"Access-Control-Allow-Methods",
                                ",".join(self.allow_methods).encode(),
                            )
                        )
                        headers.append(
                            (
                                b"Access-Control-Allow-Headers",
                                ",".join(self.allow_headers).encode(),
                            )
                        )

                    message["headers"] = headers

                await send(message)

            await handler(scope, receive, send_wrapper)

        return middleware
